searchNodes=[{"type":"module","title":"WiseGPTEx","doc":"Documentation for WiseGPTEx . This module provides functions to obtain the best completion from the OpenAI models (default: &quot;gpt-3.5-turbo&quot;) using the OpenAI API completions endpoint ( https://api.openai.com/v1/chat/completions ). The get_best_completion/2 and get_best_completion_with_resolver/2 functions take a question and an optional list of options to configure the API request. get_best_completion_with_resolver/2 also involves a secondary step to resolve the best completion among the options, which leads to an additional API call for a more accurate response. Installation Add wise_gpt_ex to your list of dependencies in mix.exs : def deps do [ { :wise_gpt_ex , &quot;~&gt; 0.5.0&quot; } ] end Add the OpenAI API key to your configuration file (e.g., config/config.exs): config :wise_gpt_ex , :openai_api_key , &quot;your_openai_api_key&quot; Add the Anthropic API key to your configuration file (e.g., config/config.exs): config :wise_gpt_ex , :anthropic_api_key , &quot;your_anthropic_api_key&quot; Examples Basic usage: iex&gt; WiseGPTEx . get_best_completion ( &quot;What is the capital of France?&quot; ) { :ok , &quot;Paris&quot; } iex&gt; WiseGPTEx . get_best_completion_with_resolver ( &quot;What is the capital of France?&quot; ) { :ok , &quot;Paris&quot; } Using a raw completion: iex&gt; messages = [ ...&gt; %{ &quot;role&quot; =&gt; &quot;system&quot; , &quot;content&quot; =&gt; &quot;You are High School Geography Teacher&quot; } , ...&gt; %{ &quot;role&quot; =&gt; &quot;user&quot; , &quot;content&quot; =&gt; &quot;What was the capital of France in 15th century?&quot; } ...&gt; ] ...&gt; WiseGPTEx . get_raw_completion ( messages ) { :ok , &quot;The capital of France in the 15th century was Paris.&quot; } Using all available options: iex&gt; opts = [ model : &quot;gpt-4&quot; , temperature : 0.7 , num_completions : 5 , timeout : 3_600_000 ] iex&gt; WiseGPTEx . get_best_completion ( &quot;What is the capital of France?&quot; , opts ) { :ok , &quot;Paris&quot; } iex&gt; WiseGPTEx . get_best_completion_with_resolver ( &quot;What is the capital of France?&quot; , opts ) { :ok , &quot;Paris&quot; } Note that the examples for the get_best_completion_with_resolver/2 function are similar to get_best_completion/2 . This is because the difference between these two functions is in the method of how they select the best completion, not in their usage or the nature of their inputs or outputs. The get_best_completion_with_resolver/2 function will perform an additional API call to get a more accurate completion, which can be beneficial for complex or ambiguous queries. Anthropic API usage: iex&gt; WiseGPTEx . get_anthropic_completion ( &quot;Why is the sky blue?&quot; ) { :ok , &quot;The sky is blue because... [detailed explanation]&quot; } Options The following options can be passed to the get_best_completion/2 and get_best_completion_with_resolver/2 functions: :model - The name of the model to use (default: &quot;gpt-3.5-turbo&quot;). All OpenAI models are supported. :temperature - Controls the randomness of the model's output. Higher values result in more diverse responses (default: 0.5). :num_completions - The number of completions to generate (default: 3). :timeout - The maximum time in milliseconds to wait for a response from the OpenAI API (default: 3_600_000 ms, or 60 minutes). For get_anthropic_completion/2 , the following options can be passed: :model - The version of the Claude model to use (default: &quot;claude-2&quot;). :temperature - Controls the randomness of the model's output (default: 0.1). :max_tokens_to_sample - Maximum number of tokens to generate (default: 100,000). :timeout - Maximum time in milliseconds to wait for a response (default: 3,600,000 ms, or 60 minutes).","ref":"WiseGPTEx.html"},{"type":"function","title":"WiseGPTEx.get_anthropic_completion/2","doc":"Retrieves a completion from the Anthropic API using the Claude model. Params: message : A string containing the prompt or question. opts : A keyword list of options to configure the API request. Returns: {:ok, binary()} : The completion for the given prompt. {:error, any()} : An error message in case of failure. This function provides a direct way to interact with the Anthropic API, allowing for detailed control over the completion request. Example: iex&gt; WiseGPTEx . get_anthropic_completion ( &quot;Why is the sky blue?&quot; ) { :ok , &quot;The sky is blue because... [detailed explanation]&quot; }","ref":"WiseGPTEx.html#get_anthropic_completion/2"},{"type":"function","title":"WiseGPTEx.get_best_completion/2","doc":"get_best_completion/2 attempts to answer a given question using OpenAI's completion endpoint. Params: question : a binary string containing the question to be answered opts : a keyword list of options to configure the API request Returns: {:ok, binary()} : the best completion for the given question {:error, any()} : an error message in the case of failure Example: iex&gt; WiseGPTEx . get_best_completion ( &quot;What is the capital of France?&quot; ) { :ok , &quot;Paris&quot; } Options: The function accepts the following options: :model - The name of the model to use (default: &quot;gpt-3.5-turbo&quot;). All OpenAI models are supported. :temperature - Controls the randomness of the model's output. Higher values result in more diverse responses (default: 0.5). :num_completions - The number of completions to generate (default: 3). :timeout - The maximum time in milliseconds to wait for a response from the OpenAI API (default: 3_600_000 ms, or 60 minutes).","ref":"WiseGPTEx.html#get_best_completion/2"},{"type":"function","title":"WiseGPTEx.get_best_completion_with_resolver/2","doc":"get_best_completion_with_resolver/2 is similar to get_best_completion/2 but uses a secondary step to resolve the best completion among the options. This function will perform an additional API call to get a more accurate completion, which can be beneficial for complex or ambiguous queries. Params: question : a binary string containing the question to be answered opts : a keyword list of options to configure the API request Returns: {:ok, binary()}: the best completion for the given question - : an error message in the case of failure Example: iex&gt; WiseGPTEx . get_best_completion_with_resolver ( &quot;What is the capital of France?&quot; ) { :ok , &quot;Paris&quot; } Options: The function accepts the following options: :model - The name of the model to use (default: &quot;gpt-3.5-turbo&quot;). All OpenAI models are supported. :temperature - Controls the randomness of the model's output. Higher values result in more diverse responses (default: 0.5). - :num_completions - The number of completions to generate (default: 3). - :timeout` - The maximum time in milliseconds to wait for a response from the OpenAI API (default: 3_600_000 ms, or 60 minutes).","ref":"WiseGPTEx.html#get_best_completion_with_resolver/2"},{"type":"function","title":"WiseGPTEx.get_raw_completion/2","doc":"Gets a raw completion from the OpenAI API without additional prompting. Params: messages : A list of messages to send to the API. Each message should be a map with keys &quot;role&quot; and &quot;content&quot;, for example: iex&gt; messages = [ ...&gt; %{ &quot;role&quot; =&gt; &quot;system&quot; , &quot;content&quot; =&gt; &quot;You are High School Geography Teacher&quot; } , ...&gt; %{ &quot;role&quot; =&gt; &quot;user&quot; , &quot;content&quot; =&gt; &quot;What was the capital of France in 15th century?&quot; } ...&gt; ] opts : a keyword list of options to configure the API request Returns: {:ok, binary()} : the completion for the given question {:error, any()} : an error message in the case of failure This allows you to customize the conversation sent to the API without any additional prompting added. Example: iex&gt; messages = [ ...&gt; %{ &quot;role&quot; =&gt; &quot;system&quot; , &quot;content&quot; =&gt; &quot;You are High School Geography Teacher&quot; } , ...&gt; %{ &quot;role&quot; =&gt; &quot;user&quot; , &quot;content&quot; =&gt; &quot;What was the capital of France in 15th century?&quot; } ...&gt; ] ...&gt; WiseGPTEx . get_raw_completion ( messages ) { :ok , &quot;The capital of France in the 15th century was Paris.&quot; }","ref":"WiseGPTEx.html#get_raw_completion/2"},{"type":"module","title":"WiseGPTEx.AnthropicHTTPClient","doc":"Documentation for WiseGPTEx.AnthropicHTTPClient . This module provides functionalities to interact with the Anthropic API, specifically to utilize the Claude model for generating responses. It is designed to be used internally within the WiseGPTEx module and is not intended for direct usage. Key functions include: complete/2 - Sends a prompt to the Anthropic API and receives a completion using the Claude model. It allows customization of various parameters such as model, temperature, and maximum tokens. NOTE: This module is designed to offer a straightforward interface to the Anthropic API, focusing on raw API calls with minimal additional processing. It provides direct and flexible access to the Claude model's capabilities. Refer to the Claude API documentation for more details: https://docs.anthropic.com/claude/reference/complete_post","ref":"WiseGPTEx.AnthropicHTTPClient.html"},{"type":"function","title":"WiseGPTEx.AnthropicHTTPClient.complete/2","doc":"Completes a prompt using Claude. Claude API docs: https://docs.anthropic.com/claude/reference/complete_post model - string | required The model that will complete your prompt. As we improve Claude, we develop new versions of it that you can query. This parameter controls which version of Claude answers your request. Right now we are offering two model families: Claude, and Claude Instant. You can use them by setting model to &quot;claude-2&quot; or &quot;claude-instant-1&quot;, respectively. prompt - blob | required The prompt that you want Claude to complete. For proper response generation you will need to format your prompt as follows: const userQuestion = r&quot;Why is the sky blue?&quot;; const prompt = ` Human: ${userQuestion} Assistant:`; max_tokens_to_sample - integer | required The maximum number of tokens to generate before stopping. Note that our models may stop before reaching this maximum. This parameter only specifies the absolute maximum number of tokens to generate. stop_sequences - array of strings Sequences that will cause the model to stop generating completion text. Our models stop on &quot; Human:&quot;, and may include additional built-in stop sequences in the future. By providing the stop_sequences parameter, you may include additional strings that will cause the model to stop generating. temperature - number Amount of randomness injected into the response. Defaults to 1. Ranges from 0 to 1. Use temp closer to 0 for analytical / multiple choice, and closer to 1 for creative and generative tasks. top_p - number Use nucleus sampling. In nucleus sampling, we compute the cumulative distribution over all the options for each subsequent token in decreasing probability order and cut it off once it reaches a particular probability specified by top_p. You should either alter temperature or top_p, but not both. top_k - integer Only sample from the top K options for each subsequent token. Used to remove &quot;long tail&quot; low probability responses. metadata - object An object describing metadata about the request. user_id - string An external identifier for the user who is associated with the request. This should be a uuid, hash value, or other opaque identifier. Anthropic may use this id to help detect abuse. Do not include any identifying information such as name, email address, or phone number. stream - boolean Whether to incrementally stream the response using server-sent events.","ref":"WiseGPTEx.AnthropicHTTPClient.html#complete/2"},{"type":"module","title":"WiseGPTEx.AnthropicUtils","doc":"Provides utility functions for processing responses from the Anthropic API.","ref":"WiseGPTEx.AnthropicUtils.html"},{"type":"function","title":"WiseGPTEx.AnthropicUtils.extract_complete/1","doc":"Extracts the completion from the response body of the Anthropic API. This function parses the response from the API, focusing on extracting the 'completion' field. It handles both successful and error responses, returning a string that represents the completion or an error message. In the case of successful responses, it returns the 'completion' field's content. For error responses, it returns a formatted error message. Examples Successful response: iex&gt; response = %{ ...&gt; &quot;completion&quot; =&gt; &quot; Okay, let&#39;s solve this step-by-step: \\n 42 * 38 \\n = 42 * (30 + 8) \\n = 42 * 30 + 42 * 8 \\n = 1260 + 336 \\n = 1596.&quot; , ...&gt; &quot;stop_reason&quot; =&gt; &quot;stop_sequence&quot; , ...&gt; &quot;model&quot; =&gt; &quot;claude-2.0&quot; , ...&gt; &quot;stop&quot; =&gt; &quot; \\n \\n Human:&quot; , ...&gt; &quot;log_id&quot; =&gt; &quot;00c6b4ba97de04c5329dc50c0124e4814dfc3252d751afd55bcfaad44023be6d&quot; ...&gt; } ...&gt; WiseGPTEx.AnthropicUtils . extract_complete ( response ) &quot; Okay, let&#39;s solve this step-by-step: \\n 42 * 38 \\n = 42 * (30 + 8) \\n = 42 * 30 + 42 * 8 \\n = 1260 + 336 \\n = 1596.&quot; Error response: iex&gt; error_response = %{ ...&gt; &quot;error&quot; =&gt; %{ ...&gt; &quot;type&quot; =&gt; &quot;not_found_error&quot; , ...&gt; &quot;message&quot; =&gt; &quot;Not found&quot; ...&gt; } ...&gt; } ...&gt; WiseGPTEx.AnthropicUtils . extract_complete ( error_response ) &quot;Error: %{ \\&quot; message \\&quot; =&gt; \\&quot; Not found \\&quot; , \\&quot; type \\&quot; =&gt; \\&quot; not_found_error \\&quot; }&quot;","ref":"WiseGPTEx.AnthropicUtils.html#extract_complete/1"},{"type":"module","title":"WiseGPTEx.OpenAIHTTPClient","doc":"Documentation for WiseGPTEx.OpenAIHTTPClient . This module contains functions to interact with the OpenAI API to get completions with reasoning and determine the best completion. It is used internally by the WiseGPTEx module and is not intended to be called directly by users. The main functions in this module are: get_completions_with_reasoning/2 - Retrieves completions with reasoning from the OpenAI API based on a given message and an optional list of options. get_best_completion/3 - Determines the best completion from a list of completions and an optional list of options. get_raw_completion/2 - Gets a raw completion from the API based on given messages, without additional prompting. NOTE: get_raw_completion/2 allows sending custom conversations with both role and content to the API, while the other functions add extra prompting to request reasoning, disambiguation, etc.","ref":"WiseGPTEx.OpenAIHTTPClient.html"},{"type":"function","title":"WiseGPTEx.OpenAIHTTPClient.get_best_completion/3","doc":"","ref":"WiseGPTEx.OpenAIHTTPClient.html#get_best_completion/3"},{"type":"function","title":"WiseGPTEx.OpenAIHTTPClient.get_completions_with_reasoning/2","doc":"","ref":"WiseGPTEx.OpenAIHTTPClient.html#get_completions_with_reasoning/2"},{"type":"function","title":"WiseGPTEx.OpenAIHTTPClient.get_raw_completion/2","doc":"","ref":"WiseGPTEx.OpenAIHTTPClient.html#get_raw_completion/2"},{"type":"function","title":"WiseGPTEx.OpenAIHTTPClient.get_resolver_completion/4","doc":"","ref":"WiseGPTEx.OpenAIHTTPClient.html#get_resolver_completion/4"},{"type":"module","title":"WiseGPTEx.OpenAIUtils","doc":"Provides facilities for processing some of the responses from third party APIs.","ref":"WiseGPTEx.OpenAIUtils.html"},{"type":"function","title":"WiseGPTEx.OpenAIUtils.extract_completion/1","doc":"A function that takes a list of completions and returns the content of the first one that finished. Example iex&gt; body = %{ ...&gt; &quot;choices&quot; =&gt; [ ...&gt; %{ ...&gt; &quot;finish_reason&quot; =&gt; &quot;stop&quot; , ...&gt; &quot;index&quot; =&gt; 0 , ...&gt; &quot;message&quot; =&gt; %{ ...&gt; &quot;content&quot; =&gt; &quot;Correct Answer: Option 1.&quot; , ...&gt; &quot;role&quot; =&gt; &quot;assistant&quot; ...&gt; } ...&gt; } ...&gt; ] ...&gt; } ...&gt; WiseGPTEx.OpenAIUtils . extract_completion ( body ) &quot;Correct Answer: Option 1.&quot;","ref":"WiseGPTEx.OpenAIUtils.html#extract_completion/1"},{"type":"function","title":"WiseGPTEx.OpenAIUtils.extract_completions/1","doc":"A function that takes a list of completions and returns the content of all that finished. Example iex&gt; body = %{ ...&gt; &quot;choices&quot; =&gt; [ ...&gt; %{ ...&gt; &quot;finish_reason&quot; =&gt; &quot;stop&quot; , ...&gt; &quot;index&quot; =&gt; 0 , ...&gt; &quot;message&quot; =&gt; %{ ...&gt; &quot;content&quot; =&gt; &quot;First, 456 * 23421 = 54,456.&quot; , ...&gt; &quot;role&quot; =&gt; &quot;assistant&quot; ...&gt; } ...&gt; } , ...&gt; %{ ...&gt; &quot;finish_reason&quot; =&gt; &quot;stop&quot; , ...&gt; &quot;index&quot; =&gt; 1 , ...&gt; &quot;message&quot; =&gt; %{ ...&gt; &quot;content&quot; =&gt; &quot;Second, we get the answer: 10,686,876.&quot; , ...&gt; &quot;role&quot; =&gt; &quot;assistant&quot; ...&gt; } ...&gt; } ...&gt; ] ...&gt; } ...&gt; WiseGPTEx.OpenAIUtils . extract_completions ( body ) [ &quot;First, 456 * 23421 = 54,456.&quot; , &quot;Second, we get the answer: 10,686,876.&quot; ]","ref":"WiseGPTEx.OpenAIUtils.html#extract_completions/1"},{"type":"function","title":"WiseGPTEx.OpenAIUtils.prepare_completion_options/1","doc":"A function that takes a list of completions and prepares them as answer options. Each option is prefixed with &quot;Answer Option&quot; followed by its index in the list, and all options are joined together with newline. Example iex&gt; completions = [ &quot;PHP&quot; , &quot;Elixir&quot; , &quot;Go&quot; ] ...&gt; WiseGPTEx.OpenAIUtils . prepare_completion_options ( completions ) &quot;Answer Option 1: PHP \\n Answer Option 2: Elixir \\n Answer Option 3: Go&quot;","ref":"WiseGPTEx.OpenAIUtils.html#prepare_completion_options/1"},{"type":"function","title":"WiseGPTEx.OpenAIUtils.reasoning_prompt/1","doc":"A function that takes a message and returns a prompt with a reasoning template. Example iex&gt; WiseGPTEx.OpenAIUtils . reasoning_prompt ( &quot;What is 2+2?&quot; ) &quot;Question: What is 2+2? \\n Answer: Let&#39;s work this out in a step by step way to be sure we have the right answer.&quot;","ref":"WiseGPTEx.OpenAIUtils.html#reasoning_prompt/1"},{"type":"function","title":"WiseGPTEx.OpenAIUtils.researcher_prompt/2","doc":"A function that takes a message and completions with reasoning, and returns a string prompt for a researcher. The researcher is tasked with investigating the answer options provided, listing the flaws and faulty logic of each answer option. Example iex&gt; message = &quot;What is the capital of France?&quot; ...&gt; completions_with_reasoning = &quot;Option 1: Paris - This is the correct answer. Paris is the capital of France. \\n Option 2: London - This is incorrect. London is the capital of the United Kingdom, not France.&quot; ...&gt; WiseGPTEx.OpenAIUtils . researcher_prompt ( message , completions_with_reasoning ) &quot;What is the capital of France? \\n \\n Option 1: Paris - This is the correct answer. Paris is the capital of France. \\n Option 2: London - This is incorrect. London is the capital of the United Kingdom, not France. \\n \\n You are researcher tasked with investigating the answer options provided. List the flaws and faulty logic of each answer option. Let&#39;s work this out in a step by step way to be sure we have all the errors \\n \\n **Answer format:** \\n Correct Answer: &lt;Option Number&gt; \\n **If there are multiple correct answers, pick the first one.**&quot;","ref":"WiseGPTEx.OpenAIUtils.html#researcher_prompt/2"},{"type":"function","title":"WiseGPTEx.OpenAIUtils.resolver_prompt/0","doc":"A simple function that returns a static string. This string serves as an instruction for a &quot;resolver&quot; role, outlining the tasks it needs to perform. These tasks include: Identifying the best answer from a set of options. Improving the selected answer. Printing the improved answer in its entirety. Adding a special marker to the beginning of the improved answer. The string also includes a statement encouraging a step-by-step approach to ensure the correct answer is found. This function doesn't take any arguments and always returns the same string. Example iex&gt; WiseGPTEx.OpenAIUtils . resolver_prompt ( ) &quot;You are a resolver tasked with 1) finding which of the X answer options was best 2) improving that answer, 3) printing the improved answer in full, and 4) adding to the begining of the improved answer special marker &lt;|answerstart|&gt;. Let&#39;s work this out in step by step way to be sure we have the right answer:&quot;","ref":"WiseGPTEx.OpenAIUtils.html#resolver_prompt/0"},{"type":"function","title":"WiseGPTEx.OpenAIUtils.trim_resolver_answer/1","doc":"A function that extracts the resolver's answer from a string. The answer is expected to start within the special marker &lt;|answerstart|&gt; . If these markers are not found, the function returns the original string. Example iex&gt; answer = &quot;Some text before&lt;|answerstart|&gt;Hello&quot; ...&gt; WiseGPTEx.OpenAIUtils . trim_resolver_answer ( answer ) &quot;Hello&quot;","ref":"WiseGPTEx.OpenAIUtils.html#trim_resolver_answer/1"}]